{% extends "email_layout.html" %}

{% block content %}
<h1>Saved Searches Report</h1>
<p>This is a report for search saved as "<strong>{{search.name|e}}</strong>"</p>
{% if not docs %}
<p><strong>No document found with this search</strong></p>
{% else %}
<p>Here are the found documents:
    <ul>
        {% for doc in docs %}
        <li>
            <strong>{{doc.headline|e}}</strong>
            (<a target="_blank" href="{{client_url}}/#/workspace/personal?item={{doc.guid|e}}&action=edit">
                <em>{{doc.guid|e}}</em>
            </a>)
        </li>
        {% endfor %}
    </ul>
</p>
{% endif %}
{% endblock %}
