{% block content %}{% endblock %}

<p>Regards,</p>

<p>The <em>{{app_name}}</em> Team</p>

<p>{{app_name}} is developed by <a href="https://sourcefabric.org">Sourcefabric</a> and the {{app_name}} community.</p>
